/* =================================================================== */
/* DECLARATIONS							       */
/* =================================================================== */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/* Include files needed for ADAMO and DAD use */

#include "cfortran.h"
#include "partap.h"
#include "dad.h"

/* Structs for all tables you want to use */

#include "mcEvKey.h"
#include "mcEvent.h"
#include "mcTrack.h"

/* Finally declare some variables of your own */

	#define		maxtracks 4
	int		ntracks;
	int		use_key;	/* use as logical, 0=F 1=T */
	int		i, iopt, igaf, idfl, iok, ievent, dadflags, dummy;
	float		momentum[maxtracks];
	char		ckey[81], cline[81], cfile[81], cdriver[5],
			dummystring[81];

/* Start of program */

	void main()
	{

/* =================================================================== */
/* ASK USER TO SELECT OPTION: TYPE OF FILE TO READ		       */
/* =================================================================== */        

	printf("\n Please select type of file to read:");
        printf("\n      0:   GAF in IE format");
        printf("\n      1:   GAF in FZ format");
        printf("\n      2:   DAD file (WFIL) via dadOpenFile");
        printf("\n      3:   DAD file (WFIL) via dadDoConnect");
        printf("\n      4:   DAD file (WFIL) via dadHost, dadConnect");
        printf("\n      5:   DAD file (WFIL) via dadInit, dadConnect");
        printf("\n      6:   DAD file (WPIP) via dadDoConnect \n");

        scanf("%d", &iopt);
        if (iopt < 0 || iopt > 6)
        {
           printf("\n Invalid selection \n");
           exit(0);
        }

        printf("\n Do you wish to use key tables? (y or n) \n");
        scanf("%s", cline);
        if ( strcmp(cline, "n") == 0 || strcmp(cline, "N") == 0 )
           use_key = 0;
        else if ( strcmp(cline, "y") == 0 || strcmp(cline, "Y") == 0 )
           use_key = 1;
        else
        {
           printf("\n You lose!  Neener neener neener...\n");
           exit(0);
        }

/* The input filenames we choose follow certain conventions */

/* ... IE format carries .ie as extension */
        if (iopt == 0)
           strcpy(cfile, "toyMC.ie");

/* ... FZ files usually carry .events extensions */
        else if (iopt == 1)
           strcpy(cfile, "toyMC.events");

/* ... DAD files are often called .devents (or .dfil) */
        else if (iopt < 5)
           strcpy(cfile, "toyMC.devents");

/* ... pipes are often called .pipe */
        else
        {
           strcpy(cfile, "toyMC.pipe");
        }

	printf("\n ---> Reading from %s", cfile);

/* =================================================================== */
/* INITIALIZE ADAMO                                                    */
/* =================================================================== */

/* The INITAP routine created by MAD initializes ADAMO and also calls 
   the CREOBJ routine to set up your personal data structures. */

        INITAP();

/* =================================================================== */
/* OPEN AN EXISTING GAF FILE                                           */
/* =================================================================== */

/* Set up the appropriate DAD flags for writing */

        if (use_key == 1)
        {
           C2FCBSTR("mcEvKey", ckey, 0);
           dadflags = CAC_READ + CAC_KEYTABLE;
        }
        else
        {
           C2FCBSTR(" ", ckey, 0);
           dadflags = CAC_READ;
        }

/* OPTIONS 0-1: Open a new GAF in IE or FZ format using OPEGAF
   ----------- */

        if (iopt <= 1)
        {
           if (iopt == 0)
              strcpy(cdriver, "IE");
           else 
              strcpy(cdriver, "FZ,FILFOR=EXCH");
           sprintf(dummystring, "NAME=%s,DRIVER=%s", cfile, cdriver);
           C2FCBSTR(dummystring, cline, 0);
           OPEGAF (igaf,cline,iok);
        }

/* OPTION 2: Open a DAD file for reading via dadOpenFile
   -------- */

        else if (iopt == 2)
           dadOpenFile (
                        &igaf,           /* identifier for the new file */
                        cfile,          /* file name */
                        "mcEvents",     /* dataflow name */
                        ckey,           /* name of key table */
                        dadflags,       /* sum of flags */
                        &iok );          /* error code */

/* OPTIONS 3-6: These more generic techniques can be used to write to files,
   -----------  pipes, and servers, all termed 'hosts'. In all cases, the 
                host must first be defined, by providing this information:
                   - NAME of host or file
                   - TYPE of host: file (WFIL), pipe (WPIP), or server (INET)
                   - PORT number (for server connections only)
                   - DATAFLOW name to work with
                   - FLAGS (see variable dadflags above)
              Then, you simply 'connect' to the chosen host. */

/* OPTION 3: Open a DAD file for reading using dadDoConnect.
   --------  The dadDoConnect routine provides a ONE-SHOT way 
             to simultaneously define the host and connect to it. */

        else if (iopt == 3)
           dadDoConnect (
                        &igaf,           /* identifier for the new file */
                        cfile,          /* file name */
                        "RFIL",         /* connection type */
                        0,              /* port (only needed for servers */
                        "mcEvents",     /* dataflow name */
                        ckey,           /* name of key table */
                        dadflags,       /* sum of flags */
                        &iok );          /* error code */

/* OPTION 4: Open a DAD file for reading using dadHost and dadConnect.
   --------  The host definition is performed here by a call to dadHost. */

        else if (iopt == 4)
        {

/* ... first, define the host */
           dadHost (
                   cfile,               /* file name */
                   "RFIL",              /* connection type */
                   0,                   /* port #, 0 for local connection */
                   "mcEvents",          /* dataflow name */
                   dadflags);           /* sum of flags */

/* ... then open the connection */
          dadConnect (
                     "mcEvents",        /* dataflow name */
                     &igaf,              /* identifier for new file */
                     &iok );             /* error code */
        }

/* OPTION 5: Open a DAD file for reading using dadInit and dadConnect.
   --------  The host definitions here are supplied by an EXTERNAL FILE, 
             named dadinit.cnf, which you must provide. For this program
             to work, your dadinit.cnf file must contain the line:
                        toyMC.devents RFIL 0 mcEvents rk
             The information is the same as that in the dadHost call.
             'rk' is shorthand for the dadflags codes (see the dad manual). */

        else if (iopt == 5)
        {

/* ... read in the host definitions from dadinit.cnf */
           dadInit();

/* ... then open the connection */
           dadConnect (
                     "mcEvents",        /* dataflow name */
                     &igaf,              /* identifier for new file */
                     &iok );             /* error code */
        }

/* OPTION 6: Open a DAD PIPE for writing using dadDoConnect.
   --------  Practically identical to OPTION 3. You can also use dadOpenPipe, 
             which allows for split pipes! (see the dad manual) */

        else if (iopt == 6)
           dadDoConnect (
                        &igaf,           /* identifier for the new file */
                        cfile,          /* file name */
                        "RPIP",         /* connection type */
                        0,              /* port (only needed for servers */
                        "mcEvents",     /* dataflow name */
                        ckey,           /* name of key table */
                        dadflags,       /* sum of flags */
                        &iok );          /* error code */

/* Check that the output file was opened successfully. */

        if (iok != 0)
        {
           printf("\n Failed to open output file. \n");
           exit(0);
        }

/* =================================================================== */
/*      **********    START LOOP OVER EVENTS    **********             */
/* =================================================================== */

	while (1)
	{

/* =================================================================== */
/*  READ IN THE NEXT RECORD					       */
/* =================================================================== */

/* OPTIONS 0-1: Retrieve the key table information for the next record 
   -----------  using NEXGAF. Next, check for EOF. Finally, actually
	        load the record into ADAMO memory using FETGAF. */

	if (iopt <= 1)
	{

/* ... pull key table info for the next record into the struct */
	   NEXGAF (igaf, mcEvKey, iok);

/* ... leave the loop if at end of file (EOF return code is -1) */
	   if (iok != 0)
	      goto endloop;

/* ... if this is the first event, obtain dataflow name from entry
   ... cName of key table, to prove this is working */
	   if (mcEvKey.iEvent == 1)
	   {
	      FCB2CSTR(mcEvKey.cName, dummystring, 0);
	      printf("\n mcEvKey.cName = %s", dummystring);
	      printf(" (should be the dataflow name, mcEvents)");
	      idfl = GETTDF (dummystring);
	   }

/* ... call FETGAF to read the tables from this record into memory.
   ... Note: 'this record' = that currently stored in the key table WIN.
   ... Since we just loaded this WIN with NEXGAF, we will be getting the 
   ... NEXT record, as planned. */
	   FETGAF (igaf, mcEvKey, idfl, iok);

	}

/* OPTIONS 2-6: Use dadRead to retrieve the next record. It automatically
   -----------  stores all tables into ADAMO memory. (All except the key
	        table, which is anyway not a 'real' table). */

	else
	{

/* ... load tables from next record into memory using dadRead */
	   dadRead (igaf, &iok);

/* ... leave the loop if at end of file */
	   if (iok != 0)
	      goto endloop;

/* ... The key table info for this record has been stored in a dad-internal
   ... area. If we want to use it, we have to explicitly copy this info
   ... into our struct using dadGetRecordKey */
	   if (use_key == 1)
	      dadGetRecordKey (igaf, &mcEvKey, &iok);

/* ... if this is the first event, obtain dataflow name from entry
   ... cName of key table, to prove this is working */
	   if (mcEvKey.iEvent == 1)
	   {
	      FCB2CSTR(mcEvKey.cName, dummystring, 0);
	      printf("\n mcEvKey.cName = %s", dummystring);
	      printf(" (should be the dataflow name, mcEvents)");
	      idfl = GETTDF (dummystring);
	   }
	}

/* =================================================================== */
/* RETRIEVE AND PRINT SOME INFORMATION FROM TABLES		       */
/* =================================================================== */

/* Retrieve some info from mcEvent and mcTrack */

	FETTAB (mcEvent,ID,1);
	ntracks = COUTAB(mcTrack);
	if (ntracks > maxtracks)
	{
	   dummy = maxtracks;
	   printf("\n ERROR: more than %d tracks in mcTrack", dummy);
	   ntracks = maxtracks;
	}
	for (i = 1; i <= ntracks; i++)
	{
	   FETTAB (mcTrack,ID,i);
	   momentum[i-1] = mcTrack.P;
	}

/* And print it out, to make sure all is working
   Note: the event number comes from the key table */

	printf("\n EVENT %d: Q2 = %f,", mcEvKey.iEvent, mcEvent.Q2);
	printf(" %d tracks with p =", COUTAB(mcTrack) );
	for (i = 1; i <= ntracks; i++)
	{
	   printf("   %f", momentum[i-1]);
	}

/* =================================================================== */
/*     **********    END LOOP OVER RECORDS   **********		       */
/* =================================================================== */

/* Go back for the next record (end of infinite while loop) */

	}

/* We end up here when EOF is reached */

	endloop: ;

/* =================================================================== */
/*   CLOSE THE GAF						       */
/* =================================================================== */

/* OPTIONS 0-1: Close the GAF using CLOGAF
   ----------- */

	if (iopt <= 1)
	   CLOGAF (igaf,iok);

/* OPTION 2: Close the DAD file using dadCloseFile (note, you can also
   --------  use dadDisconnect ... dadCloseFile is just a special case) */

        else if (iopt == 2)
           dadCloseFile (igaf, &iok);

/* OPTIONS 3-6: Close the generic DAD host using dadDisconnect
   ----------- */

        else
           dadDisconnect (igaf, &iok);

/* End of program */

	printf("\n");
	}
