/* =================================================================== */
/* DECLARATIONS							       */
/* =================================================================== */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/* Include files needed for ADAMO and DAD use */

#include "cfortran.h"
#include "partap.h"
#include "dad.h"

/* Structs for all tables you want to use */

#include "mcEvKey.h"
#include "mcEvent.h"
#include "mcTrack.h"

/* Finally declare some variables of your own */

	const float ebeam = 	27.5;
	const float q2min =	1.0;
	const float q2max =	20.0;
	const int nevents =	10;
	int use_key;	/* use as logical, 0=F, 1=T */
	int i, iopt, igaf, idfl, iok, ievent, dadflags, dummy;
	char ckey[81], cline[81], cfile[81], cdriver[5], dummystring[81];	

/* ... declare external function */
	float notrandom( void );

/* Start of program */
	void main()
	{

/* =================================================================== */
/*  ASK USER TO SELECT OPTION: TYPE OF FILE TO WRITE		       */
/* =================================================================== */

	printf("\n Please select type of file to write:");
	printf("\n	0:   GAF in IE format");
	printf("\n	1:   GAF in FZ format");
	printf("\n	2:   DAD file (WFIL) via dadOpenFile");
	printf("\n	3:   DAD file (WFIL) via dadDoConnect");
	printf("\n	4:   DAD file (WFIL) via dadHost, dadConnect");
	printf("\n	5:   DAD file (WFIL) via dadInit, dadConnect");
	printf("\n	6:   DAD file (WPIP) via dadDoConnect \n");
	
	scanf("%d", &iopt);
	if (iopt < 0 || iopt > 6)
	{
	   printf("\n Invalid selection \n");
	   exit(0);
	}

	printf("\n Do you wish to use key tables? (y or n) \n");
	scanf("%s", cline);
	if ( strcmp(cline, "n") == 0 || strcmp(cline, "N") == 0 )
	   use_key = 0;
	else if ( strcmp(cline, "y") == 0 || strcmp(cline, "Y") == 0 )
	   use_key = 1;
	else
	{
	   printf("\n You lose!  Neener neener neener...\n");
	   exit(0);
	}

/* The output filenames we choose follow certain conventions */

/* ... IE format carries .ie as extension */
	if (iopt == 0)
	   strcpy(cfile, "toyMC.ie");

/* ... FZ files usually carry .events extensions */
	else if (iopt == 1)
	   strcpy(cfile, "toyMC.events");

/* ... DAD files are often called .devents (or .dfil) */
	else if (iopt < 5)
	   strcpy(cfile, "toyMC.devents");

/* ... pipes are often called .pipe */
	else
	{
	   strcpy(cfile, "toyMC.pipe");
	   printf("\n ---> Writing to pipe %s", cfile);
	}

/* =================================================================== */
/* INITIALIZE ADAMO						       */
/* =================================================================== */

/* The INITAP routine created by MAD initializes ADAMO and also calls 
   the CREOBJ routine to set up your personal data structures. */

	INITAP();

/* =================================================================== */
/* OPEN A NEW GAF FILE						       */
/* =================================================================== */

/* Set up the appropriate DAD flags for writing */

	if (use_key == 1)
	{
	   C2FCBSTR("mcEvKey", ckey, 0);
	   dadflags = CAC_WRITE + CAC_KEYTABLE;
	}
/* ........ NOTE: the key table field MUST be left blank here! */

	else
	{
	   C2FCBSTR(" ", ckey, 0);
	   dadflags = CAC_WRITE;
	}

/* OPTIONS 0-1: Create a new GAF in IE or FZ format using CREGAF
   ----------- */

	if (iopt <= 1)
	{
	   if (iopt == 0)
	      strcpy(cdriver, "IE");
	   else 
	      strcpy(cdriver, "FZ,FILFOR=EXCH");
	   sprintf(dummystring, "NAME=%s,DRIVER=%s", cfile, cdriver);
	   C2FCBSTR(dummystring, cline, 0);
	   CREGAF (igaf,mcEvKey,cline,iok);
	}

/* OPTION 2: Open a DAD file for writing via dadOpenFile
   -------- */

	else if (iopt == 2)
	   dadOpenFile (
			&igaf,		/* identifier for the new file */
			cfile,		/* file name */
			"mcEvents",	/* dataflow name */
			ckey,		/* name of key table */
			dadflags,	/* sum of flags */
			&iok );		/* error code */

/* OPTIONS 3-6: These more generic techniques can be used to write to files,
   -----------  pipes, and servers, all termed 'hosts'. In all cases, the 
                host must first be defined, by providing this information:
		   - NAME of host or file
		   - TYPE of host: file (WFIL), pipe (WPIP), or server (INET)
		   - PORT number (for server connections only)
		   - DATAFLOW name to work with
		   - FLAGS (see variable dadflags above)
              Then, you simply 'connect' to the chosen host. */

/* OPTION 3: Open a DAD file for writing using dadDoConnect.
   --------  The dadDoConnect routine provides a ONE-SHOT way 
 	     to simultaneously define the host and connect to it. */

	else if (iopt == 3)
	   dadDoConnect (
			&igaf,		/* identifier for the new file */
			cfile,		/* file name */
			"WFIL",		/* connection type */
			0,		/* port (only needed for servers */
			"mcEvents",	/* dataflow name */
			ckey,		/* name of key table */
			dadflags,	/* sum of flags */
			&iok );		/* error code */

/* OPTION 4: Open a DAD file for writing using dadHost and dadConnect.
   --------  The host definition is performed here by a call to dadHost.
	     Note that I cannot get keytable writing to work here ... */

	else if (iopt == 4)
	{

/* ... first, define the host */
	   dadHost (
		   cfile,		/* file name */
		   "WFIL",		/* connection type */
		   0,			/* port #, 0 for local connection */
		   "mcEvents",		/* dataflow name */
		   dadflags);		/* sum of flags */

/* ... then open the connection */
	  dadConnect (
		     "mcEvents",	/* dataflow name */
		     &igaf,		/* identifier for new file */
		     &iok );		/* error code */
	}

/* OPTION 5: Open a DAD file for writing using dadInit and dadConnect.
   --------  The host definitions here are supplied by an EXTERNAL FILE, 
	     named dadinit.cnf, which you must provide. For this program
	     to work, your dadinit.cnf file must contain the line:
		        toyMC.devents WFIL 0 mcEvents wk
	     The information is the same as that in the dadHost call.
	     'wk' is shorthand for the dadflags codes (see the dad manual).
	     Note that I cannot get keytable writing to work here ... */

	else if (iopt == 5)
	{

/* ... read in the host definitions from dadinit.cnf */
	   dadInit();

/* ... then open the connection */
	   dadConnect (
		     "mcEvents",	/* dataflow name */
		     &igaf,		/* identifier for new file */
		     &iok );		/* error code */
	}

/* OPTION 6: Open a DAD PIPE for writing using dadDoConnect.
   --------  Practically identical to OPTION 3. You can also use dadOpenPipe, 
	     which allows for split pipes! (see the dad manual) */

	else if (iopt == 6)
	   dadDoConnect (
			&igaf,		/* identifier for the new file */
			cfile,		/* file name */
			"WPIP",		/* connection type */
			0,		/* port (only needed for servers */
			"mcEvents",	/* dataflow name */
			ckey,		/* name of key table */
			dadflags,	/* sum of flags */
			&iok );		/* error code */

/* Check that the output file was opened successfully. */

	if (iok != 0)
	{
	   printf("\n Failed to open output file. \n");
	   exit(0);
	}

/* =================================================================== */
/*      **********    START LOOP OVER EVENTS    **********	       */
/* =================================================================== */

	for (ievent = 1; ievent <= nevents; ievent++)
	{

	   printf("\n          <<<<< WRITING EVENT NUMBER");
	   printf(" %d >>>>>", ievent);

/* =================================================================== */
/* GENERATE THE EVENT						       */
/* =================================================================== */

/* Clear tables. */
/* NOTE: when applied to a dataflow, CLETAB only clears the tables that 
   are located IMMEDIATELY inside it ... CLETAB will NOT navigate down a 
   tree of dataflows. */

	   dummy = GETTDF("mcEvents");
	   CLETAB (dummy);

/* Fill the mcEvent table */

	   mcEvent.ID = 1;
	   mcEvent.Nu = notrandom()*ebeam;
	   mcEvent.Q2 = q2min + (q2max-q2min)*notrandom();
	   INSTAB (mcEvent);

/* Fill mcTrack with 4 random particle tracks */

	   for (i = 1; i <= 4; i++)
	   {
	      mcTrack.ID = NEXT;
	      mcTrack.P = mcEvent.Nu * notrandom();
	      INSTAB (mcTrack);
	   }

/* =================================================================== */
/* STORE THE EVENT RECORD TO THE GAF				       */
/* =================================================================== */

/* Fill the key table struct to identify this record (record = 'event' 
   here)  Note: There is no need to load the key table to memory with 
   INSTAB. */

	   mcEvKey.ID = 1;
	   mcEvKey.iRun = 1;
	   mcEvKey.iEvent = ievent;
	   C2FCBSTR("mcEvents", mcEvKey.cName, 0);

/* ... DAD only: we must explicitly copy our struct to the dad-internal 
   ... version */
	   if (iopt > 1 && use_key == 1)
	     dadFillRecordKey (igaf, &mcEvKey, &iok);

/* OPTIONS 0-1: Store the record in the GAF using INSGAF
   ----------- */

	   if (iopt <= 1)
	     INSGAF (igaf,mcEvKey,iok);

/* OPTIONS 2-6: Store the record in the DAD file using dadWrite
   ----------- */

	   else
	     dadWrite (igaf, &iok);

/* =================================================================== */
/*      **********    END LOOP OVER EVENTS   **********		       */
/* =================================================================== */

	}

/* =================================================================== */
/* CLOSE THE GAF						       */
/* =================================================================== */

/* OPTIONS 0-1: Close the GAF using CLOGAF
   ----------- */

	if (iopt <= 1)
	   CLOGAF (igaf,iok);

/* OPTION 2: Close the DAD file using dadCloseFile (note, you can also
   --------  use dadDisconnect ... dadCloseFile is just a special case) */

	else if (iopt == 2)
	   dadCloseFile (igaf, &iok);

/* OPTIONS 3-6: Close the generic DAD host using dadDisconnect
   ----------- */

	else
	   dadDisconnect (igaf,&iok);

/* Tell the user what we did */

	printf("\n ---> Wrote file: %s \n", cfile);

/* End of program */

	}

/* ================================================================= */
/* CODING EXAMPLE: the NOTRANDOM number generator :)                 */
/* ================================================================= */

/* Before defining the function, declare a few more variables */

        const float increment = 0.09;
        float current_number = 0.;
	double junk;

        float notrandom(void)
        {
           current_number = current_number + increment;
	   current_number = modf(current_number, &junk);
	   return current_number;
	}
